@@ -28,6 +28,8 @@ from breezy import (
     lockdir,
     rio,
     shelf,
+    )
+from breezy.bzr import (
     tag as _mod_tag,
     )
 """)
@@ -450,6 +452,36 @@ class BzrBranch(Branch, _RelockDebugMixin):
             reconciler = BranchReconciler(self, thorough=thorough)
             return reconciler.reconcile()
 
+    def set_reference_info(self, file_id, branch_location, path=None):
+        """Set the branch location to use for a tree reference."""
+        raise errors.UnsupportedOperation(self.set_reference_info, self)
+
+    def get_reference_info(self, file_id, path=None):
+        """Get the tree_path and branch_location for a tree reference."""
+        raise errors.UnsupportedOperation(self.get_reference_info, self)
+
+    def reference_parent(self, file_id, path, possible_transports=None):
+        """Return the parent branch for a tree-reference.
+
+        :param path: The path of the nested tree in the tree
+        :return: A branch associated with the nested tree
+        """
+        try:
+            branch_location = self.get_reference_info(file_id)[0]
+        except errors.UnsupportedOperation:
+            branch_location = None
+        if branch_location is None:
+            try:
+                return Branch.open_from_transport(
+                    self.controldir.root_transport.clone(path),
+                    possible_transports=possible_transports)
+            except errors.NotBranchError:
+                return None
+        return Branch.open(
+            urlutils.join(
+                urlutils.strip_segment_parameters(self.user_url), branch_location),
+            possible_transports=possible_transports)
+
 
 class BzrBranch8(BzrBranch):
     """A branch that stores tree-reference locations."""
@@ -518,15 +550,15 @@ class BzrBranch8(BzrBranch):
     def _set_all_reference_info(self, info_dict):
         """Replace all reference info stored in a branch.
 
-        :param info_dict: A dict of {file_id: (tree_path, branch_location)}
+        :param info_dict: A dict of {file_id: (branch_location, tree_path)}
         """
         s = BytesIO()
         writer = rio.RioWriter(s)
-        for tree_path, (branch_location, file_id) in viewitems(info_dict):
-            stanza = rio.Stanza(tree_path=tree_path,
+        for file_id, (branch_location, tree_path) in viewitems(info_dict):
+            stanza = rio.Stanza(file_id=file_id,
                                 branch_location=branch_location)
-            if file_id is not None:
-                stanza.add('file_id', file_id)
+            if tree_path is not None:
+                stanza.add('tree_path', tree_path)
             writer.write_stanza(stanza)
         with self.lock_write():
             self._transport.put_bytes('references', s.getvalue())
@@ -540,52 +572,39 @@ class BzrBranch8(BzrBranch):
         with self.lock_read():
             if self._reference_info is not None:
                 return self._reference_info
-            with self._transport.get('references') as rio_file:
-                stanzas = rio.read_stanzas(rio_file)
-                info_dict = {
-                    s['tree_path']: (
-                        s['branch_location'],
-                        s['file_id'].encode('ascii')
-                        if 'file_id' in s else None)
-                    for s in stanzas}
+            try:
+                with self._transport.get('references') as rio_file:
+                    stanzas = rio.read_stanzas(rio_file)
+                    info_dict = {
+                        s['file_id'].encode('utf-8'): (
+                            s['branch_location'],
+                            s['tree_path'] if 'tree_path' in s else None)
+                        for s in stanzas}
+            except errors.NoSuchFile:
+                info_dict = {}
             self._reference_info = info_dict
             return info_dict
 
-    def set_reference_info(self, tree_path, branch_location, file_id=None):
+    def set_reference_info(self, file_id, branch_location, tree_path=None):
         """Set the branch location to use for a tree reference.
 
-        :param tree_path: The path of the tree reference in the tree.
         :param branch_location: The location of the branch to retrieve tree
             references from.
         :param file_id: The file-id of the tree reference.
+        :param tree_path: The path of the tree reference in the tree.
         """
         info_dict = self._get_all_reference_info()
-        info_dict[tree_path] = (branch_location, file_id)
+        info_dict[file_id] = (branch_location, tree_path)
         if branch_location is None:
-            del info_dict[tree_path]
+            del info_dict[file_id]
         self._set_all_reference_info(info_dict)
 
-    def get_reference_info(self, path):
+    def get_reference_info(self, file_id):
         """Get the tree_path and branch_location for a tree reference.
 
-        :return: a tuple of (branch_location, file_id)
+        :return: a tuple of (branch_location, tree_path)
         """
-        return self._get_all_reference_info().get(path, (None, None))
-
-    def reference_parent(self, path, file_id=None, possible_transports=None):
-        """Return the parent branch for a tree-reference file_id.
-
-        :param file_id: The file_id of the tree reference
-        :param path: The path of the file_id in the tree
-        :return: A branch associated with the file_id
-        """
-        branch_location = self.get_reference_info(path)[0]
-        if branch_location is None:
-            return Branch.reference_parent(self, path, file_id,
-                                           possible_transports)
-        branch_location = urlutils.join(self.user_url, branch_location)
-        return Branch.open(branch_location,
-                           possible_transports=possible_transports)
+        return self._get_all_reference_info().get(file_id, (None, None))
 
     def set_push_location(self, location):
         """See Branch.set_push_location."""
@@ -686,15 +705,12 @@ class BzrBranch8(BzrBranch):
 class BzrBranch7(BzrBranch8):
     """A branch with support for a fallback repository."""
 
-    def set_reference_info(self, tree_path, branch_location, file_id=None):
-        Branch.set_reference_info(self, file_id, tree_path, branch_location)
-
-    def get_reference_info(self, path):
-        Branch.get_reference_info(self, path)
-
-    def reference_parent(self, path, file_id=None, possible_transports=None):
-        return Branch.reference_parent(
-            self, path, file_id, possible_transports)
+    def set_reference_info(self, file_id, branch_location, tree_path=None):
+        super(BzrBranch7, self).set_reference_info(
+            file_id, branch_location, tree_path)
+        format_string = BzrBranchFormat8.get_format_string()
+        mutter('Upgrading branch to format %r', format_string)
+        self._transport.put_bytes('format', format_string)
 
 
 class BzrBranch6(BzrBranch7):
@@ -862,6 +878,8 @@ class BzrBranchFormat6(BranchFormatMetadir):
     def supports_set_append_revisions_only(self):
         return True
 
+    supports_reference_locations = True
+
 
 class BzrBranchFormat8(BranchFormatMetadir):
     """Metadir format supporting storing locations of subtree branches."""
@@ -945,7 +963,9 @@ class BzrBranchFormat7(BranchFormatMetadir):
         """See breezy.branch.BranchFormat.make_tags()."""
         return _mod_tag.BasicTags(branch)
 
-    supports_reference_locations = False
+    # This is a white lie; as soon as you set a reference location, we upgrade
+    # you to BzrBranchFormat8.
+    supports_reference_locations = True
 
 
 class BranchReferenceFormat(BranchFormatMetadir):
@@ -971,7 +991,7 @@ class BranchReferenceFormat(BranchFormatMetadir):
     def get_reference(self, a_controldir, name=None):
         """See BranchFormat.get_reference()."""
         transport = a_controldir.get_branch_transport(None, name=name)
-        url = urlutils.split_segment_parameters(a_controldir.user_url)[0]
+        url = urlutils.strip_segment_parameters(a_controldir.user_url)
         return urlutils.join(
             url, transport.get_bytes('location').decode('utf-8'))
 
@@ -1012,10 +1032,10 @@ class BranchReferenceFormat(BranchFormatMetadir):
 
     def _make_reference_clone_function(format, a_branch):
         """Create a clone() routine for a branch dynamically."""
-        def clone(to_bzrdir, revision_id=None,
-                  repository_policy=None):
+        def clone(to_bzrdir, revision_id=None, repository_policy=None, name=None,
+                  tag_selector=None):
             """See Branch.clone()."""
-            return format.initialize(to_bzrdir, target_branch=a_branch)
+            return format.initialize(to_bzrdir, target_branch=a_branch, name=name)
             # cannot obey revision_id limits when cloning a reference ...
             # FIXME RBC 20060210 either nuke revision_id for clone, or
             # emit some sort of warning/error to the caller ?!
