@@ -18,6 +18,10 @@
 
 from __future__ import absolute_import
 
+from .. import (
+    errors,
+    )
+
 from ..lazy_import import lazy_import
 lazy_import(globals(), """
 import time
@@ -25,7 +29,6 @@ import time
 from breezy import (
     controldir,
     debug,
-    errors,
     osutils,
     revision as _mod_revision,
     trace,
@@ -267,8 +270,6 @@ class RepositoryFormatKnitPack4(RepositoryFormatPack):
 class RepositoryFormatKnitPack5(RepositoryFormatPack):
     """Repository that supports external references to allow stacking.
 
-    New in release 1.6.
-
     Supports external lookups, which results in non-truncated ghosts after
     reconcile compared to pack-0.92 formats.
     """
@@ -306,8 +307,6 @@ class RepositoryFormatKnitPack5(RepositoryFormatPack):
 class RepositoryFormatKnitPack5RichRoot(RepositoryFormatPack):
     """A repository with rich roots and stacking.
 
-    New in release 1.6.1.
-
     Supports stacking on other repositories, allowing data to be accessed
     without being stored locally.
     """
@@ -347,8 +346,6 @@ class RepositoryFormatKnitPack5RichRoot(RepositoryFormatPack):
 class RepositoryFormatKnitPack5RichRootBroken(RepositoryFormatPack):
     """A repository with rich roots and external references.
 
-    New in release 1.6.
-
     Supports external lookups, which results in non-truncated ghosts after
     reconcile compared to pack-0.92 formats.
 
@@ -684,7 +681,7 @@ class KnitPacker(Packer):
                 else:
                     df, _ = knit._parse_record_header(key, raw_data)
                     df.close()
-                pos, size = writer.add_bytes_record(raw_data, names)
+                pos, size = writer.add_bytes_record([raw_data], len(raw_data), names)
                 write_index.add_node(key, eol_flag + b"%d %d" % (pos, size))
                 pb.update("Copied record", record_index)
                 record_index += 1
@@ -736,7 +733,7 @@ class KnitPacker(Packer):
                     # check the header only
                     df, _ = knit._parse_record_header(key, raw_data)
                     df.close()
-                pos, size = writer.add_bytes_record(raw_data, names)
+                pos, size = writer.add_bytes_record([raw_data], len(raw_data), names)
                 write_index.add_node(key, eol_flag + b"%d %d" %
                                      (pos, size), references)
                 pb.update("Copied record", record_index)
@@ -1089,8 +1086,8 @@ class KnitReconcilePacker(KnitPacker):
                     raise errors.BzrError('Mismatched key parent %r:%r' %
                                           (key, parent_keys))
                 parents.append(parent_key[1])
-            text_lines = osutils.split_lines(next(repo.texts.get_record_stream(
-                [key], 'unordered', True)).get_bytes_as('fulltext'))
+            text_lines = next(repo.texts.get_record_stream(
+                [key], 'unordered', True)).get_bytes_as('lines')
             output_texts.add_lines(key, parent_keys, text_lines,
                                    random_id=True, check_content=False)
         # 5) check that nothing inserted has a reference outside the keyspace.
