@@ -696,7 +696,7 @@ class _DebugCounter(object):
         self.counts = weakref.WeakKeyDictionary()
         client._SmartClient.hooks.install_named_hook(
             'call', self.increment_call_count, 'hpss call counter')
-        breezy.get_global_state().cleanups.add_cleanup(self.flush_all)
+        breezy.get_global_state().exit_stack.callback(self.flush_all)
 
     def track(self, medium):
         """Start tracking calls made to a medium.
