@@ -29,7 +29,6 @@ lazy_import(globals(), """
 from breezy import (
     annotate,
     bencode,
-    errors,
     graph as _mod_graph,
     osutils,
     multiparent,
@@ -43,6 +42,9 @@ from breezy.bzr import (
     knit,
     )
 """)
+from .. import (
+    errors,
+    )
 from ..registry import Registry
 from ..sixish import (
     BytesIO,
@@ -54,26 +56,26 @@ from ..textmerge import TextMerge
 
 
 adapter_registry = Registry()
-adapter_registry.register_lazy(('knit-delta-gz', 'fulltext'), 'breezy.bzr.knit',
-                               'DeltaPlainToFullText')
-adapter_registry.register_lazy(('knit-ft-gz', 'fulltext'), 'breezy.bzr.knit',
-                               'FTPlainToFullText')
 adapter_registry.register_lazy(('knit-annotated-delta-gz', 'knit-delta-gz'),
                                'breezy.bzr.knit', 'DeltaAnnotatedToUnannotated')
-adapter_registry.register_lazy(('knit-annotated-delta-gz', 'fulltext'),
-                               'breezy.bzr.knit', 'DeltaAnnotatedToFullText')
 adapter_registry.register_lazy(('knit-annotated-ft-gz', 'knit-ft-gz'),
                                'breezy.bzr.knit', 'FTAnnotatedToUnannotated')
-adapter_registry.register_lazy(('knit-annotated-ft-gz', 'fulltext'),
-                               'breezy.bzr.knit', 'FTAnnotatedToFullText')
-# adapter_registry.register_lazy(('knit-annotated-ft-gz', 'chunked'),
-#     'breezy.bzr.knit', 'FTAnnotatedToChunked')
+for target_storage_kind in ('fulltext', 'chunked', 'lines'):
+    adapter_registry.register_lazy(('knit-delta-gz', target_storage_kind), 'breezy.bzr.knit',
+                                   'DeltaPlainToFullText')
+    adapter_registry.register_lazy(('knit-ft-gz', target_storage_kind), 'breezy.bzr.knit',
+                                   'FTPlainToFullText')
+    adapter_registry.register_lazy(('knit-annotated-ft-gz', target_storage_kind),
+                                   'breezy.bzr.knit', 'FTAnnotatedToFullText')
+    adapter_registry.register_lazy(('knit-annotated-delta-gz', target_storage_kind),
+                                   'breezy.bzr.knit', 'DeltaAnnotatedToFullText')
 
 
 class ContentFactory(object):
     """Abstract interface for insertion and retrieval from a VersionedFile.
 
     :ivar sha1: None, or the sha1 of the content fulltext.
+    :ivar size: None, or the size of the content fulltext.
     :ivar storage_kind: The native storage kind of this factory. One of
         'mpdiff', 'knit-annotated-ft', 'knit-annotated-delta', 'knit-ft',
         'knit-delta', 'fulltext', 'knit-annotated-ft-gz',
@@ -88,6 +90,7 @@ class ContentFactory(object):
     def __init__(self):
         """Create a ContentFactory."""
         self.sha1 = None
+        self.size = None
         self.storage_kind = None
         self.key = None
         self.parents = None
@@ -101,6 +104,7 @@ class ChunkedContentFactory(ContentFactory):
     satisfies this, as does a list of lines.
 
     :ivar sha1: None, or the sha1 of the content fulltext.
+    :ivar size: None, or the size of the content fulltext.
     :ivar storage_kind: The native storage kind of this factory. Always
         'chunked'
     :ivar key: The key of this content. Each key is a tuple with a single
@@ -108,24 +112,40 @@ class ChunkedContentFactory(ContentFactory):
     :ivar parents: A tuple of parent keys for self.key. If the object has
         no parent information, None (as opposed to () for an empty list of
         parents).
+    :ivar chunks_are_lines: Whether chunks are lines.
      """
 
-    def __init__(self, key, parents, sha1, chunks):
+    def __init__(self, key, parents, sha1, chunks, chunks_are_lines=None):
         """Create a ContentFactory."""
         self.sha1 = sha1
+        self.size = sum(map(len, chunks))
         self.storage_kind = 'chunked'
         self.key = key
         self.parents = parents
         self._chunks = chunks
+        self._chunks_are_lines = chunks_are_lines
 
     def get_bytes_as(self, storage_kind):
         if storage_kind == 'chunked':
             return self._chunks
         elif storage_kind == 'fulltext':
             return b''.join(self._chunks)
+        elif storage_kind == 'lines':
+            if self._chunks_are_lines:
+                return self._chunks
+            return list(osutils.chunks_to_lines(self._chunks))
         raise errors.UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 
+    def iter_bytes_as(self, storage_kind):
+        if storage_kind == 'chunked':
+            return iter(self._chunks)
+        elif storage_kind == 'lines':
+            if self._chunks_are_lines:
+                return iter(self._chunks)
+            return iter(osutils.chunks_to_lines(self._chunks))
+        raise errors.UnavailableRepresentation(self.key, storage_kind,
+                                               self.storage_kind)
 
 class FulltextContentFactory(ContentFactory):
     """Static data content factory.
@@ -146,6 +166,7 @@ class FulltextContentFactory(ContentFactory):
     def __init__(self, key, parents, sha1, text):
         """Create a ContentFactory."""
         self.sha1 = sha1
+        self.size = len(text)
         self.storage_kind = 'fulltext'
         self.key = key
         self.parents = parents
@@ -158,6 +179,49 @@ class FulltextContentFactory(ContentFactory):
             return self._text
         elif storage_kind == 'chunked':
             return [self._text]
+        elif storage_kind == 'lines':
+            return osutils.split_lines(self._text)
+        raise errors.UnavailableRepresentation(self.key, storage_kind,
+                                               self.storage_kind)
+
+    def iter_bytes_as(self, storage_kind):
+        if storage_kind == 'chunked':
+            return iter([self._text])
+        elif storage_kind == 'lines':
+            return iter(osutils.split_lines(self._text))
+        raise errors.UnavailableRepresentation(self.key, storage_kind,
+                                               self.storage_kind)
+
+
+class FileContentFactory(ContentFactory):
+    """File-based content factory.
+    """
+
+    def __init__(self, key, parents, fileobj, sha1=None, size=None):
+        self.key = key
+        self.parents = parents
+        self.file = fileobj
+        self.storage_kind = 'file'
+        self.sha1 = sha1
+        self.size = size
+
+    def get_bytes_as(self, storage_kind):
+        self.file.seek(0)
+        if storage_kind == 'fulltext':
+            return self.file.read()
+        elif storage_kind == 'chunked':
+            return list(osutils.file_iterator(self.file))
+        elif storage_kind == 'lines':
+            return list(self.file.readlines())
+        raise errors.UnavailableRepresentation(self.key, storage_kind,
+                                               self.storage_kind)
+
+    def iter_bytes_as(self, storage_kind):
+        self.file.seek(0)
+        if storage_kind == 'chunked':
+            return osutils.file_iterator(self.file)
+        elif storage_kind == 'lines':
+            return self.file
         raise errors.UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 
@@ -175,6 +239,7 @@ class AbsentContentFactory(ContentFactory):
     def __init__(self, key):
         """Create a ContentFactory."""
         self.sha1 = None
+        self.size = None
         self.storage_kind = 'absent'
         self.key = key
         self.parents = None
@@ -185,6 +250,12 @@ class AbsentContentFactory(ContentFactory):
                          ' code does not handle if it is missing.'
                          % (self.key,))
 
+    def iter_bytes_as(self, storage_kind):
+        raise ValueError('A request was made for key: %s, but that'
+                         ' content is not available, and the calling'
+                         ' code does not handle if it is missing.'
+                         % (self.key,))
+
 
 class AdapterFactory(ContentFactory):
     """A content factory to adapt between key prefix's."""
@@ -723,6 +794,15 @@ class RecordingVersionedFilesDecorator(object):
         return self._backing_vf.add_lines(key, parents, lines, parent_texts,
                                           left_matching_blocks, nostore_sha, random_id, check_content)
 
+    def add_content(self, factory, parent_texts=None,
+                    left_matching_blocks=None, nostore_sha=None, random_id=False,
+                    check_content=True):
+        self.calls.append(("add_content", factory, parent_texts,
+                           left_matching_blocks, nostore_sha, random_id, check_content))
+        return self._backing_vf.add_content(
+            factory, parent_texts, left_matching_blocks, nostore_sha,
+            random_id, check_content)
+
     def check(self):
         self._backing_vf.check()
 
@@ -978,6 +1058,38 @@ class VersionedFiles(object):
         """
         raise NotImplementedError(self.add_lines)
 
+    def add_content(self, factory, parent_texts=None,
+                    left_matching_blocks=None, nostore_sha=None, random_id=False,
+                    check_content=True):
+        """Add a text to the store from a chunk iterable.
+
+        :param key: The key tuple of the text to add. If the last element is
+            None, a CHK string will be generated during the addition.
+        :param parents: The parents key tuples of the text to add.
+        :param chunk_iter: An iterable over bytestrings.
+        :param parent_texts: An optional dictionary containing the opaque
+            representations of some or all of the parents of version_id to
+            allow delta optimisations.  VERY IMPORTANT: the texts must be those
+            returned by add_lines or data corruption can be caused.
+        :param left_matching_blocks: a hint about which areas are common
+            between the text and its left-hand-parent.  The format is
+            the SequenceMatcher.get_matching_blocks format.
+        :param nostore_sha: Raise ExistingContent and do not add the lines to
+            the versioned file if the digest of the lines matches this.
+        :param random_id: If True a random id has been selected rather than
+            an id determined by some deterministic process such as a converter
+            from a foreign VCS. When True the backend may choose not to check
+            for uniqueness of the resulting key within the versioned file, so
+            this should only be done when the result is expected to be unique
+            anyway.
+        :param check_content: If True, the lines supplied are verified to be
+            bytestrings that are correctly formed lines.
+        :return: The text sha1, the number of bytes in the text, and an opaque
+                 representation of the inserted version which can be provided
+                 back to future add_lines calls in the parent_texts dictionary.
+        """
+        raise NotImplementedError(self.add_content)
+
     def add_mpdiffs(self, records):
         """Add mpdiffs to this VersionedFile.
 
@@ -996,13 +1108,11 @@ class VersionedFiles(object):
                                   if not mpvf.has_version(p))
         # It seems likely that adding all the present parents as fulltexts can
         # easily exhaust memory.
-        chunks_to_lines = osutils.chunks_to_lines
         for record in self.get_record_stream(needed_parents, 'unordered',
                                              True):
             if record.storage_kind == 'absent':
                 continue
-            mpvf.add_version(chunks_to_lines(record.get_bytes_as('chunked')),
-                             record.key, [])
+            mpvf.add_version(record.get_bytes_as('lines'), record.key, [])
         for (key, parent_keys, expected_sha1, mpdiff), lines in zip(
                 records, mpvf.get_line_list(versions)):
             if len(parent_keys) == 1:
@@ -1201,6 +1311,18 @@ class ThunkedVersionedFiles(VersionedFiles):
         self._mapper = mapper
         self._is_locked = is_locked
 
+    def add_content(self, factory, parent_texts=None,
+                    left_matching_blocks=None, nostore_sha=None, random_id=False):
+        """See VersionedFiles.add_content()."""
+        lines = factory.get_bytes_as('lines')
+        return self.add_lines(
+            factory.key, factory.parents, lines,
+            parent_texts=parent_texts,
+            left_matching_blocks=left_matching_blocks,
+            nostore_sha=nostore_sha,
+            random_id=random_id,
+            check_content=True)
+
     def add_lines(self, key, parents, lines, parent_texts=None,
                   left_matching_blocks=None, nostore_sha=None, random_id=False,
                   check_content=True):
@@ -1478,6 +1600,10 @@ class _PlanMergeVersionedFile(VersionedFiles):
             ver_a, base, self, (self._file_id,), graph).plan_merge()
         return _PlanLCAMerge._subtract_plans(list(old_plan), list(new_plan))
 
+    def add_content(self, factory):
+        return self.add_lines(
+            factory.key, factory.parents, factory.get_bytes_as('lines'))
+
     def add_lines(self, key, parents, lines):
         """See VersionedFiles.add_lines
 
@@ -1502,7 +1628,9 @@ class _PlanMergeVersionedFile(VersionedFiles):
                 lines = self._lines[key]
                 parents = self._parents[key]
                 pending.remove(key)
-                yield ChunkedContentFactory(key, parents, None, lines)
+                yield ChunkedContentFactory(
+                    key, parents, None, lines,
+                    chunks_are_lines=True)
         for versionedfile in self.fallback_versionedfiles:
             for record in versionedfile.get_record_stream(
                     pending, 'unordered', True):
@@ -1731,9 +1859,9 @@ class VirtualVersionedFiles(VersionedFiles):
             if lines is not None:
                 if not isinstance(lines, list):
                     raise AssertionError
-                yield ChunkedContentFactory((k,), None,
-                                            sha1=osutils.sha_strings(lines),
-                                            chunks=lines)
+                yield ChunkedContentFactory(
+                    (k,), None, sha1=osutils.sha_strings(lines),
+                    chunks=lines, chunks_are_lines=True)
             else:
                 yield AbsentContentFactory((k,))
 
