@@ -129,6 +129,18 @@ class ControlDir(ControlComponent):
         """
         return list(self.get_branches().values())
 
+    def branch_names(self):
+        """List all branch names in this control directory.
+
+        :return: List of branch names
+        """
+        try:
+            self.get_branch_reference()
+        except (errors.NotBranchError, errors.NoRepositoryPresent):
+            return []
+        else:
+            return [""]
+
     def get_branches(self):
         """Get all branches in this control directory, as a dictionary.
 
@@ -152,7 +164,7 @@ class ControlDir(ControlComponent):
         this in the future - for instance to make bzr talk with svn working
         trees.
         """
-        raise NotImplementedError(self.is_control_filename)
+        return self._format.is_control_filename(filename)
 
     def needs_format_conversion(self, format=None):
         """Return true if this controldir needs convert_format run on it.
@@ -400,7 +412,8 @@ class ControlDir(ControlComponent):
         raise NotImplementedError(self.sprout)
 
     def push_branch(self, source, revision_id=None, overwrite=False,
-                    remember=False, create_prefix=False, lossy=False):
+                    remember=False, create_prefix=False, lossy=False,
+                    tag_selector=None):
         """Push the source branch into this ControlDir."""
         br_to = None
         # If we can open a branch, use its direct repository, otherwise see
@@ -424,7 +437,9 @@ class ControlDir(ControlComponent):
                 # revision
                 revision_id = source.last_revision()
             repository_to.fetch(source.repository, revision_id=revision_id)
-            br_to = source.sprout(self, revision_id=revision_id, lossy=lossy)
+            br_to = source.sprout(
+                self, revision_id=revision_id, lossy=lossy,
+                tag_selector=tag_selector)
             if source.get_push_location() is None or remember:
                 # FIXME: Should be done only if we succeed ? -- vila 2012-01-18
                 source.set_push_location(br_to.base)
@@ -444,17 +459,19 @@ class ControlDir(ControlComponent):
                 tree_to = self.open_workingtree()
             except errors.NotLocalUrl:
                 push_result.branch_push_result = source.push(
-                    br_to, overwrite, stop_revision=revision_id, lossy=lossy)
+                    br_to, overwrite, stop_revision=revision_id, lossy=lossy,
+                    tag_selector=tag_selector)
                 push_result.workingtree_updated = False
             except errors.NoWorkingTree:
                 push_result.branch_push_result = source.push(
-                    br_to, overwrite, stop_revision=revision_id, lossy=lossy)
+                    br_to, overwrite, stop_revision=revision_id, lossy=lossy,
+                    tag_selector=tag_selector)
                 push_result.workingtree_updated = None  # Not applicable
             else:
                 with tree_to.lock_write():
                     push_result.branch_push_result = source.push(
                         tree_to.branch, overwrite, stop_revision=revision_id,
-                        lossy=lossy)
+                        lossy=lossy, tag_selector=tag_selector)
                     tree_to.update()
                 push_result.workingtree_updated = True
             push_result.old_revno = push_result.branch_push_result.old_revno
@@ -493,7 +510,7 @@ class ControlDir(ControlComponent):
         raise NotImplementedError(self.check_conversion_target)
 
     def clone(self, url, revision_id=None, force_new_repo=False,
-              preserve_stacking=False):
+              preserve_stacking=False, tag_selector=None):
         """Clone this controldir and its contents to url verbatim.
 
         :param url: The url create the clone at.  If url's last component does
@@ -509,11 +526,13 @@ class ControlDir(ControlComponent):
         return self.clone_on_transport(_mod_transport.get_transport(url),
                                        revision_id=revision_id,
                                        force_new_repo=force_new_repo,
-                                       preserve_stacking=preserve_stacking)
+                                       preserve_stacking=preserve_stacking,
+                                       tag_selector=tag_selector)
 
     def clone_on_transport(self, transport, revision_id=None,
                            force_new_repo=False, preserve_stacking=False, stacked_on=None,
-                           create_prefix=False, use_existing_dir=True, no_tree=False):
+                           create_prefix=False, use_existing_dir=True, no_tree=False,
+                           tag_selector=None):
         """Clone this controldir and its contents to transport verbatim.
 
         :param transport: The transport for the location to produce the clone
@@ -561,7 +580,8 @@ class ControlDir(ControlComponent):
             recurse = True
             try:
                 controldir = klass.open_from_transport(current_transport)
-            except (errors.NotBranchError, errors.PermissionDenied):
+            except (errors.NotBranchError, errors.PermissionDenied,
+                    errors.UnknownFormatError):
                 pass
             else:
                 recurse, value = evaluate(controldir)
@@ -788,7 +808,7 @@ class ControlDir(ControlComponent):
             a_transport = new_t
 
     @classmethod
-    def open_tree_or_branch(klass, location):
+    def open_tree_or_branch(klass, location, name=None):
         """Return the branch and working tree at a location.
 
         If there is no tree at the location, tree will be None.
@@ -797,7 +817,7 @@ class ControlDir(ControlComponent):
         :return: (tree, branch)
         """
         controldir = klass.open(location)
-        return controldir._get_tree_branch()
+        return controldir._get_tree_branch(name=name)
 
     @classmethod
     def open_containing_tree_or_branch(klass, location,
@@ -1027,12 +1047,6 @@ class ControlDirFormat(object):
     _default_format = None
     """The default format used for new control directories."""
 
-    _server_probers = []
-    """The registered server format probers, e.g. RemoteBzrProber.
-
-    This is a list of Prober-derived classes.
-    """
-
     _probers = []
     """The registered format probers, e.g. BzrProber.
 
@@ -1123,24 +1137,13 @@ class ControlDirFormat(object):
         """
         klass._probers.remove(prober)
 
-    @classmethod
-    def register_server_prober(klass, prober):
-        """Register a control format prober for client-server environments.
-
-        These probers will be used before ones registered with
-        register_prober.  This gives implementations that decide to the
-        chance to grab it before anything looks at the contents of the format
-        file.
-        """
-        klass._server_probers.append(prober)
-
     def __str__(self):
         # Trim the newline
         return self.get_format_description().rstrip()
 
     @classmethod
     def all_probers(klass):
-        return klass._server_probers + klass._probers
+        return klass._probers
 
     @classmethod
     def known_formats(klass):
@@ -1155,7 +1158,9 @@ class ControlDirFormat(object):
     def find_format(klass, transport, probers=None):
         """Return the format present at transport."""
         if probers is None:
-            probers = klass.all_probers()
+            probers = sorted(
+                klass.all_probers(),
+                key=lambda prober: prober.priority(transport))
         for prober_kls in probers:
             prober = prober_kls()
             try:
@@ -1246,6 +1251,22 @@ class ControlDirFormat(object):
         """
         raise NotImplementedError(self.supports_transport)
 
+    @classmethod
+    def is_control_filename(klass, filename):
+        """True if filename is the name of a path which is reserved for
+        controldirs.
+
+        :param filename: A filename within the root transport of this
+            controldir.
+
+        This is true IF and ONLY IF the filename is part of the namespace reserved
+        for bzr control dirs. Currently this is the '.bzr' directory in the root
+        of the root_transport. it is expected that plugins will need to extend
+        this in the future - for instance to make bzr talk with svn working
+        trees.
+        """
+        raise NotImplementedError(self.is_control_filename)
+
 
 class Prober(object):
     """Abstract class that can be used to detect a particular kind of
@@ -1259,8 +1280,8 @@ class Prober(object):
     probers that detect .bzr/ directories and Bazaar smart servers,
     respectively.
 
-    Probers should be registered using the register_server_prober or
-    register_prober methods on ControlDirFormat.
+    Probers should be registered using the register_prober methods on
+    ControlDirFormat.
     """
 
     def probe_transport(self, transport):
@@ -1284,6 +1305,21 @@ class Prober(object):
         """
         raise NotImplementedError(klass.known_formats)
 
+    @classmethod
+    def priority(klass, transport):
+        """Priority of this prober.
+
+        A lower value means the prober gets checked first.
+
+        Other conventions:
+
+        -10: This is a "server" prober
+        0: No priority set
+        10: This is a regular file-based prober
+        100: This is a prober for an unsupported format
+        """
+        return 0
+
 
 class ControlDirFormatInfo(object):
 
@@ -1461,8 +1497,13 @@ class RepoInitHookParams(object):
 
 def is_control_filename(filename):
     """Check if filename is used for control directories."""
-    # TODO(jelmer): Allow registration by other VCSes
-    return filename == '.bzr'
+    # TODO(jelmer): Instead, have a function that returns all control
+    # filenames.
+    for key, format in format_registry.items():
+        if format().is_control_filename(filename):
+            return True
+    else:
+        return False
 
 
 class RepositoryAcquisitionPolicy(object):
