@@ -18,37 +18,32 @@ from __future__ import absolute_import
 
 from breezy import (
     osutils,
+    trace,
     )
 from .sixish import (
     StringIO,
     )
-from .trace import is_quiet
+from .tree import TreeChange
 
 
 class TreeDelta(object):
     """Describes changes from one tree to another.
 
-    Contains seven lists:
+    Contains seven lists with TreeChange objects.
 
     added
-        (path, id, kind)
     removed
-        (path, id, kind)
     renamed
-        (oldpath, newpath, id, kind, text_modified, meta_modified)
+    copied
     kind_changed
-        (path, id, old_kind, new_kind)
     modified
-        (path, id, kind, text_modified, meta_modified)
     unchanged
-        (path, id, kind)
     unversioned
-        (path, None, kind)
 
     Each id is listed only once.
 
-    Files that are both modified and renamed are listed only in
-    renamed, with the text_modified flag true. The text_modified
+    Files that are both modified and renamed or copied are listed only in
+    renamed or copied, with the text_modified flag true. The text_modified
     applies either to the content of the file or the target of the
     symbolic link, depending of the kind of file.
 
@@ -63,6 +58,7 @@ class TreeDelta(object):
         self.added = []
         self.removed = []
         self.renamed = []
+        self.copied = []
         self.kind_changed = []
         self.modified = []
         self.unchanged = []
@@ -75,6 +71,7 @@ class TreeDelta(object):
         return self.added == other.added \
             and self.removed == other.removed \
             and self.renamed == other.renamed \
+            and self.copied == other.copied \
             and self.modified == other.modified \
             and self.unchanged == other.unchanged \
             and self.kind_changed == other.kind_changed \
@@ -85,32 +82,20 @@ class TreeDelta(object):
 
     def __repr__(self):
         return "TreeDelta(added=%r, removed=%r, renamed=%r," \
-            " kind_changed=%r, modified=%r, unchanged=%r," \
-            " unversioned=%r)" % (self.added,
-                                  self.removed, self.renamed, self.kind_changed, self.modified,
-                                  self.unchanged, self.unversioned)
+            " copied=%r, kind_changed=%r, modified=%r, unchanged=%r," \
+            " unversioned=%r)" % (
+                self.added, self.removed, self.renamed, self.copied,
+                self.kind_changed, self.modified, self.unchanged,
+                self.unversioned)
 
     def has_changed(self):
         return bool(self.modified
                     or self.added
                     or self.removed
                     or self.renamed
+                    or self.copied
                     or self.kind_changed)
 
-    def touches_file_id(self, file_id):
-        """Return True if file_id is modified by this delta."""
-        for l in self.added, self.removed, self.modified:
-            for v in l:
-                if v[1] == file_id:
-                    return True
-        for v in self.renamed:
-            if v[2] == file_id:
-                return True
-        for v in self.kind_changed:
-            if v[1] == file_id:
-                return True
-        return False
-
     def get_changes_as_text(self, show_ids=False, show_unchanged=False,
                             short_status=False):
         output = StringIO()
@@ -125,56 +110,62 @@ def _compare_trees(old_tree, new_tree, want_unchanged, specific_files,
     delta = TreeDelta()
     # mutter('start compare_trees')
 
-    for (file_id, path, content_change, versioned, parent_id, name, kind,
-         executable) in new_tree.iter_changes(old_tree, want_unchanged,
-                                              specific_files, extra_trees=extra_trees,
-                                              require_versioned=require_versioned,
-                                              want_unversioned=want_unversioned):
-        if versioned == (False, False):
-            delta.unversioned.append((path[1], None, kind[1]))
+    for change in new_tree.iter_changes(
+            old_tree, want_unchanged, specific_files, extra_trees=extra_trees,
+            require_versioned=require_versioned,
+            want_unversioned=want_unversioned):
+        if change.versioned == (False, False):
+            delta.unversioned.append(change)
             continue
-        if not include_root and (None, None) == parent_id:
+        if not include_root and (None, None) == change.parent_id:
             continue
-        fully_present = tuple((versioned[x] and kind[x] is not None) for
-                              x in range(2))
+        fully_present = tuple(
+            (change.versioned[x] and change.kind[x] is not None)
+            for x in range(2))
         if fully_present[0] != fully_present[1]:
             if fully_present[1] is True:
-                delta.added.append((path[1], file_id, kind[1]))
+                delta.added.append(change)
             else:
-                delta.removed.append((path[0], file_id, kind[0]))
+                if change.kind[0] == 'symlink' and not new_tree.supports_symlinks():
+                    trace.warning(
+                        'Ignoring "%s" as symlinks '
+                        'are not supported on this filesystem.' % (change.path[0],))
+                else:
+                    delta.removed.append(change)
         elif fully_present[0] is False:
-            delta.missing.append((path[1], file_id, kind[1]))
-        elif name[0] != name[1] or parent_id[0] != parent_id[1]:
-            # If the name changes, or the parent_id changes, we have a rename
+            delta.missing.append(change)
+        elif change.name[0] != change.name[1] or change.parent_id[0] != change.parent_id[1]:
+            # If the name changes, or the parent_id changes, we have a rename or copy
             # (if we move a parent, that doesn't count as a rename for the
             # file)
-            delta.renamed.append((path[0],
-                                  path[1],
-                                  file_id,
-                                  kind[1],
-                                  content_change,
-                                  (executable[0] != executable[1])))
-        elif kind[0] != kind[1]:
-            delta.kind_changed.append((path[1], file_id, kind[0], kind[1]))
-        elif content_change or executable[0] != executable[1]:
-            delta.modified.append((path[1], file_id, kind[1],
-                                   content_change,
-                                   (executable[0] != executable[1])))
+            if change.copied:
+                delta.copied.append(change)
+            else:
+                delta.renamed.append(change)
+        elif change.kind[0] != change.kind[1]:
+            delta.kind_changed.append(change)
+        elif change.changed_content or change.executable[0] != change.executable[1]:
+            delta.modified.append(change)
         else:
-            delta.unchanged.append((path[1], file_id, kind[1]))
+            delta.unchanged.append(change)
 
-    delta.removed.sort()
-    delta.added.sort()
-    delta.renamed.sort()
+    def change_key(change):
+        if change.path[0] is None:
+            path = change.path[1]
+        else:
+            path = change.path[0]
+        return (path, change.file_id)
 
-    def missing_key(change):
-        return (change[0] or '', change[1])
-    delta.missing.sort(key=missing_key)
+    delta.removed.sort(key=change_key)
+    delta.added.sort(key=change_key)
+    delta.renamed.sort(key=change_key)
+    delta.copied.sort(key=change_key)
+    delta.missing.sort(key=change_key)
     # TODO: jam 20060529 These lists shouldn't need to be sorted
     #       since we added them in alphabetical order.
-    delta.modified.sort()
-    delta.unchanged.sort()
-    delta.unversioned.sort()
+    delta.modified.sort(key=change_key)
+    delta.unchanged.sort(key=change_key)
+    delta.unversioned.sort(key=change_key)
 
     return delta
 
@@ -238,22 +229,22 @@ class _ChangeReporter(object):
             self.output("Operating on whole tree but only reporting on "
                         "'%s' view." % (self.view_name,))
 
-    def report(self, file_id, paths, versioned, renamed, modified, exe_change,
+    def report(self, paths, versioned, renamed, copied, modified, exe_change,
                kind):
         """Report one change to a file
 
-        :param file_id: The file_id of the file
         :param path: The old and new paths as generated by Tree.iter_changes.
         :param versioned: may be 'added', 'removed', 'unchanged', or
             'unversioned.
         :param renamed: may be True or False
+        :param copied: may be True or False
         :param modified: may be 'created', 'deleted', 'kind changed',
             'modified' or 'unchanged'.
         :param exe_change: True if the execute bit has changed
         :param kind: A pair of file kinds, as generated by Tree.iter_changes.
             None indicates no file present.
         """
-        if is_quiet():
+        if trace.is_quiet():
             return
         if paths[1] == '' and versioned == 'added' and self.suppress_root_add:
             return
@@ -272,13 +263,13 @@ class _ChangeReporter(object):
         # ( the path is different OR
         #   the kind is different)
         if (versioned == 'unchanged' and
-                (renamed or modified == 'kind changed')):
-            if renamed:
-                # on a rename, we show old and new
+                (renamed or copied or modified == 'kind changed')):
+            if renamed or copied:
+                # on a rename or copy, we show old and new
                 old_path, path = paths
             else:
-                # if it's not renamed, we're showing both for kind changes
-                # so only show the new path
+                # if it's not renamed or copied, we're showing both for kind
+                # changes so only show the new path
                 old_path, path = paths[1], paths[1]
             # if the file is not missing in the source, we show its kind
             # when we show two paths.
@@ -294,6 +285,8 @@ class _ChangeReporter(object):
             path = paths[1]
         if renamed:
             rename = "R"
+        elif copied:
+            rename = "C"
         else:
             rename = self.versioned_map[versioned]
         # we show the old kind on the new path when the content is deleted.
@@ -328,40 +321,45 @@ def report_changes(change_iterator, reporter):
         }
 
     def path_key(change):
-        if change[1][0] is not None:
-            path = change[1][0]
+        if change.path[0] is not None:
+            path = change.path[0]
         else:
-            path = change[1][1]
+            path = change.path[1]
         return osutils.splitpath(path)
-    for (file_id, path, content_change, versioned, parent_id, name, kind,
-         executable) in sorted(change_iterator, key=path_key):
+    for change in sorted(change_iterator, key=path_key):
         exe_change = False
         # files are "renamed" if they are moved or if name changes, as long
         # as it had a value
-        if None not in name and None not in parent_id and\
-                (name[0] != name[1] or parent_id[0] != parent_id[1]):
-            renamed = True
+        if None not in change.name and None not in change.parent_id and\
+                (change.name[0] != change.name[1] or change.parent_id[0] != change.parent_id[1]):
+            if change.copied:
+                copied = True
+                renamed = False
+            else:
+                renamed = True
+                copied = False
         else:
+            copied = False
             renamed = False
-        if kind[0] != kind[1]:
-            if kind[0] is None:
+        if change.kind[0] != change.kind[1]:
+            if change.kind[0] is None:
                 modified = "created"
-            elif kind[1] is None:
+            elif change.kind[1] is None:
                 modified = "deleted"
             else:
                 modified = "kind changed"
         else:
-            if content_change:
+            if change.changed_content:
                 modified = "modified"
-            elif kind[0] is None:
+            elif change.kind[0] is None:
                 modified = "missing"
             else:
                 modified = "unchanged"
-            if kind[1] == "file":
-                exe_change = (executable[0] != executable[1])
-        versioned_change = versioned_change_map[versioned]
-        reporter.report(file_id, path, versioned_change, renamed, modified,
-                        exe_change, kind)
+            if change.kind[1] == "file":
+                exe_change = (change.executable[0] != change.executable[1])
+        versioned_change = versioned_change_map[change.versioned]
+        reporter.report(change.path, versioned_change, renamed, copied, modified,
+                        exe_change, change.kind)
 
 
 def report_delta(to_file, delta, short_status=False, show_ids=False,
@@ -381,8 +379,8 @@ def report_delta(to_file, delta, short_status=False, show_ids=False,
     :param indent: Added at the beginning of all output lines (for merged
         revisions).
 
-    :param predicate: A callable receiving a path and a file id and
-        returning True if the path should be displayed.
+    :param predicate: A callable receiving a path returning True if the path
+        should be displayed.
 
     :param classify: Add special symbols to indicate file kind.
     """
@@ -399,19 +397,22 @@ def report_delta(to_file, delta, short_status=False, show_ids=False,
         return path
 
     def show_more_renamed(item):
-        (oldpath, file_id, kind,
-         text_modified, meta_modified, newpath) = item
-        dec_new_path = decorate_path(newpath, kind, meta_modified)
+        dec_new_path = decorate_path(item.path[1], item.kind[1], item.meta_modified())
         to_file.write(' => %s' % dec_new_path)
-        if text_modified or meta_modified:
-            extra_modified.append((newpath, file_id, kind,
-                                   text_modified, meta_modified))
+        if item.changed_content or item.meta_modified():
+            extra_modified.append(TreeChange(
+                item.file_id, (item.path[1], item.path[1]),
+                item.changed_content,
+                item.versioned,
+                (item.parent_id[1], item.parent_id[1]),
+                (item.name[1], item.name[1]),
+                (item.kind[1], item.kind[1]),
+                item.executable))
 
     def show_more_kind_changed(item):
-        (path, file_id, old_kind, new_kind) = item
-        to_file.write(' (%s => %s)' % (old_kind, new_kind))
+        to_file.write(' (%s => %s)' % (item.kind[0], item.kind[1]))
 
-    def show_path(path, file_id, kind, meta_modified,
+    def show_path(path, kind, meta_modified,
                   default_format, with_file_id_format):
         dec_path = decorate_path(path, kind, meta_modified)
         if show_ids:
@@ -431,35 +432,33 @@ def report_delta(to_file, delta, short_status=False, show_ids=False,
             prefix = indent + prefix + '  '
 
             for item in files:
-                path, file_id, kind = item[:3]
-                if (predicate is not None and not predicate(path, file_id)):
+                if item.path[0] is None:
+                    path = item.path[1]
+                    kind = item.kind[1]
+                else:
+                    path = item.path[0]
+                    kind = item.kind[0]
+                if predicate is not None and not predicate(path):
                     continue
                 if not header_shown and not short_status:
                     to_file.write(indent + long_status_name + ':\n')
                     header_shown = True
-                meta_modified = None
-                if len(item) == 5:
-                    meta_modified = item[4]
-
                 to_file.write(prefix)
-                show_path(path, file_id, kind, meta_modified,
+                show_path(path, kind, item.meta_modified(),
                           default_format, with_file_id_format)
                 if show_more is not None:
                     show_more(item)
-                if show_ids:
-                    to_file.write(' %s' % file_id.decode('utf-8'))
+                if show_ids and getattr(item, 'file_id', None):
+                    to_file.write(' %s' % item.file_id.decode('utf-8'))
                 to_file.write('\n')
 
     show_list(delta.removed, 'removed', 'D')
     show_list(delta.added, 'added', 'A')
     show_list(delta.missing, 'missing', '!')
     extra_modified = []
-    # Reorder delta.renamed tuples so that all lists share the same
-    # order for their 3 first fields and that they also begin like
-    # the delta.modified tuples
-    renamed = [(p, i, k, tm, mm, np)
-               for p, np, i, k, tm, mm in delta.renamed]
-    show_list(renamed, 'renamed', 'R', with_file_id_format='%s',
+    show_list(delta.renamed, 'renamed', 'R', with_file_id_format='%s',
+              show_more=show_more_renamed)
+    show_list(delta.copied, 'copied', 'C', with_file_id_format='%s',
               show_more=show_more_renamed)
     show_list(delta.kind_changed, 'kind changed', 'K',
               with_file_id_format='%s',
