@@ -41,12 +41,15 @@ from ..mapping import default_mapping
 from ..remote import (
     split_git_url,
     parse_git_error,
+    parse_git_hangup,
     HeadUpdateFailed,
     RemoteGitError,
     RemoteGitBranchFormat,
+    _git_url_and_path_from_transport,
     )
 
 from dulwich import porcelain
+from dulwich.errors import HangupException
 from dulwich.repo import Repo as GitRepo
 
 
@@ -134,6 +137,53 @@ class ParseGitErrorTests(TestCase):
         self.assertEqual(e.path, 'porridge/gaduhistory.git')
         self.assertEqual(e.extra, ': denied to jelmer')
 
+    def test_invalid_repo_name(self):
+        e = parse_git_error(
+            "url",
+            """Gregwar/fatcat/tree/debian is not a valid repository name
+Email support@github.com for help
+""")
+        self.assertIsInstance(e, NotBranchError)
+
+
+class ParseHangupTests(TestCase):
+
+    def setUp(self):
+        super(ParseHangupTests, self).setUp()
+        try:
+            HangupException(['foo'])
+        except TypeError:
+            self.skipTest('dulwich version too old')
+
+    def test_not_set(self):
+        self.assertIsInstance(
+            parse_git_hangup('http://', HangupException()), HangupException)
+
+    def test_single_line(self):
+        self.assertEqual(
+            RemoteGitError('foo bar'),
+            parse_git_hangup('http://', HangupException(['foo bar'])))
+
+    def test_multi_lines(self):
+        self.assertEqual(
+            RemoteGitError('foo bar\nbla bla'),
+            parse_git_hangup(
+                'http://', HangupException(['foo bar', 'bla bla'])))
+
+    def test_filter_boring(self):
+        self.assertEqual(
+            RemoteGitError('foo bar'), parse_git_hangup('http://', HangupException(
+                ['=======', 'foo bar', '======'])))
+
+    def test_permission_denied(self):
+        self.assertEqual(
+            PermissionDenied('http://', 'You are not allowed to push code to this project.'),
+            parse_git_hangup(
+                'http://',
+                HangupException(
+                    ['=======',
+                     'You are not allowed to push code to this project.', '', '======'])))
+
 
 class TestRemoteGitBranchFormat(TestCase):
 
@@ -348,6 +398,42 @@ class PushToRemoteBase(object):
              },
             self.remote_real.get_refs())
 
+    def test_push_branch_symref(self):
+        cfg = self.remote_real.get_config()
+        cfg.set((b'core', ), b'bare', True)
+        cfg.write_to_path()
+        self.remote_real.refs.set_symbolic_ref(b'HEAD', b'refs/heads/master')
+        c1 = self.remote_real.do_commit(
+            message=b'message',
+            committer=b'committer <committer@example.com>',
+            author=b'author <author@example.com>',
+            ref=b'refs/heads/master')
+        remote = ControlDir.open(self.remote_url)
+        wt = self.make_branch_and_tree('local', format=self._from_format)
+        self.build_tree(['local/blah'])
+        wt.add(['blah'])
+        revid = wt.commit('blah')
+
+        if self._from_format == 'git':
+            result = remote.push_branch(wt.branch, overwrite=True)
+        else:
+            result = remote.push_branch(wt.branch, lossy=True, overwrite=True)
+
+        self.assertEqual(None, result.old_revno)
+        if self._from_format == 'git':
+            self.assertEqual(1, result.new_revno)
+        else:
+            self.assertIs(None, result.new_revno)
+
+        result.report(BytesIO())
+
+        self.assertEqual(
+            {
+                b'HEAD': self.remote_real.refs[b'refs/heads/master'],
+                b'refs/heads/master': self.remote_real.refs[b'refs/heads/master'],
+            },
+            self.remote_real.get_refs())
+
     def test_push_branch_new_with_tags(self):
         remote = ControlDir.open(self.remote_url)
         builder = self.make_branch_builder('local', format=self._from_format)
@@ -522,6 +608,8 @@ class RemoteControlDirTests(TestCaseWithTransport):
         self.assertEqual(
             {'': 'master', 'blah': 'blah', 'master': 'master'},
             {n: b.name for (n, b) in remote.get_branches().items()})
+        self.assertEqual(
+            set(['', 'blah', 'master']), set(remote.branch_names()))
 
     def test_remove_tag(self):
         c1 = self.remote_real.do_commit(
@@ -611,3 +699,34 @@ class RemoteControlDirTests(TestCaseWithTransport):
             author=b'author <author@example.com>')
         remote = ControlDir.open(self.remote_url)
         self.assertEqual('master', remote.open_branch().nick)
+
+
+class GitUrlAndPathFromTransportTests(TestCase):
+
+    def test_file(self):
+        split_url = _git_url_and_path_from_transport('file:///home/blah')
+        self.assertEqual(split_url.scheme, 'file')
+        self.assertEqual(split_url.path, '/home/blah')
+
+    def test_file_segment_params(self):
+        split_url = _git_url_and_path_from_transport('file:///home/blah,branch=master')
+        self.assertEqual(split_url.scheme, 'file')
+        self.assertEqual(split_url.path, '/home/blah')
+
+    def test_git_smart(self):
+        split_url = _git_url_and_path_from_transport(
+            'git://github.com/dulwich/dulwich,branch=master')
+        self.assertEqual(split_url.scheme, 'git')
+        self.assertEqual(split_url.path, '/dulwich/dulwich')
+
+    def test_https(self):
+        split_url = _git_url_and_path_from_transport(
+            'https://github.com/dulwich/dulwich')
+        self.assertEqual(split_url.scheme, 'https')
+        self.assertEqual(split_url.path, '/dulwich/dulwich')
+
+    def test_https_segment_params(self):
+        split_url = _git_url_and_path_from_transport(
+            'https://github.com/dulwich/dulwich,branch=master')
+        self.assertEqual(split_url.scheme, 'https')
+        self.assertEqual(split_url.path, '/dulwich/dulwich')
