@@ -49,14 +49,17 @@ import imp
 from importlib import util as importlib_util
 
 from breezy import (
-    config,
+    bedding,
     debug,
-    errors,
     help_topics,
     trace,
     )
 """)
 
+from . import (
+    errors,
+    )
+
 
 _MODULE_PREFIX = "breezy.plugins."
 
@@ -407,7 +410,7 @@ def _get_site_plugin_paths(sys_paths):
 
 
 def get_user_plugin_path():
-    return osutils.pathjoin(config.config_dir(), 'plugins')
+    return osutils.pathjoin(bedding.config_dir(), 'plugins')
 
 
 def record_plugin_warning(warning_message):
