@@ -59,8 +59,9 @@ import breezy.branch
 import breezy.revision
 from ... import (
     builtins,
-    errors as bazErrors,
+    errors,
     lazy_import,
+    lru_cache,
     osutils,
     progress,
     trace,
@@ -81,6 +82,8 @@ lazy_import.lazy_import(globals(),
 from fastimport import commands
 """)
 
+REVISIONS_CHUNK_SIZE = 1000
+
 
 def _get_output_stream(destination):
     if destination is None or destination == '-':
@@ -188,6 +191,7 @@ class BzrFastExporter(object):
         self.rewrite_tags = rewrite_tags
         self.no_tags = no_tags
         self.baseline = baseline
+        self.tree_cache = lru_cache.LRUCache(max_cache=20)
         self._multi_author_api_available = hasattr(breezy.revision.Revision,
                                                    'get_apparent_authors')
         self.properties_to_exclude = ['authors', 'author']
@@ -214,8 +218,8 @@ class BzrFastExporter(object):
 
     def interesting_history(self):
         if self.revision:
-            rev1, rev2 = builtins._get_revision_range(self.revision,
-                                                      self.branch, "fast-export")
+            rev1, rev2 = builtins._get_revision_range(
+                self.revision, self.branch, "fast-export")
             start_rev_id = rev1.rev_id
             end_rev_id = rev2.rev_id
         else:
@@ -230,14 +234,37 @@ class BzrFastExporter(object):
         # revisions to exclude now ...
         if start_rev_id is not None:
             self.note("Calculating the revisions to exclude ...")
-            self.excluded_revisions = set([rev_id for rev_id, _, _, _ in
-                                           self.branch.iter_merge_sorted_revisions(start_rev_id)])
+            self.excluded_revisions = set(
+                [rev_id for rev_id, _, _, _ in self.branch.iter_merge_sorted_revisions(start_rev_id)])
             if self.baseline:
                 # needed so the first relative commit knows its parent
                 self.excluded_revisions.remove(start_rev_id)
                 view_revisions.insert(0, start_rev_id)
         return list(view_revisions)
 
+    def emit_commits(self, interesting):
+        if self.baseline:
+            revobj = self.branch.repository.get_revision(interesting.pop(0))
+            self.emit_baseline(revobj, self.ref)
+        for i in range(0, len(interesting), REVISIONS_CHUNK_SIZE):
+            chunk = interesting[i:i + REVISIONS_CHUNK_SIZE]
+            history = dict(self.branch.repository.iter_revisions(chunk))
+            trees_needed = set()
+            trees = {}
+            for revid in chunk:
+                trees_needed.update(self.preprocess_commit(revid, history[revid], self.ref))
+
+            for tree in self._get_revision_trees(trees_needed):
+                trees[tree.get_revision_id()] = tree
+
+            for revid in chunk:
+                revobj = history[revid]
+                if len(revobj.parent_ids) == 0:
+                    parent = breezy.revision.NULL_REVISION
+                else:
+                    parent = revobj.parent_ids[0]
+                self.emit_commit(revobj, self.ref, trees[parent], trees[revid])
+
     def run(self):
         # Export the data
         with self.branch.repository.lock_read():
@@ -247,10 +274,7 @@ class BzrFastExporter(object):
                       self._commit_total)
             if not self.plain_format:
                 self.emit_features()
-            if self.baseline:
-                self.emit_baseline(interesting.pop(0), self.ref)
-            for revid in interesting:
-                self.emit_commit(revid, self.ref)
+            self.emit_commits(interesting)
             if self.branch.supports_tags() and not self.no_tags:
                 self.emit_tags()
 
@@ -310,7 +334,7 @@ class BzrFastExporter(object):
         try:
             if tree.kind(path) != 'directory':
                 return False
-        except bazErrors.NoSuchFile:
+        except errors.NoSuchFile:
             self.warning("Skipping empty_dir detection - no file_id for %s" %
                          (path,))
             return False
@@ -326,52 +350,52 @@ class BzrFastExporter(object):
         for feature in sorted(commands.FEATURE_NAMES):
             self.print_cmd(commands.FeatureCommand(feature))
 
-    def emit_baseline(self, revid, ref):
+    def emit_baseline(self, revobj, ref):
         # Emit a full source tree of the first commit's parent
-        revobj = self.branch.repository.get_revision(revid)
         mark = 1
-        self.revid_to_mark[revid] = mark
-        file_cmds = self._get_filecommands(
-            breezy.revision.NULL_REVISION, revid)
+        self.revid_to_mark[revobj.revision_id] = mark
+        tree_old = self.branch.repository.revision_tree(
+            breezy.revision.NULL_REVISION)
+        [tree_new] = list(self._get_revision_trees([revobj.revision_id]))
+        file_cmds = self._get_filecommands(tree_old, tree_new)
         self.print_cmd(commands.ResetCommand(ref, None))
         self.print_cmd(self._get_commit_command(ref, mark, revobj, file_cmds))
 
-    def emit_commit(self, revid, ref):
+    def preprocess_commit(self, revid, revobj, ref):
         if revid in self.revid_to_mark or revid in self.excluded_revisions:
             return
-
-        # Get the Revision object
-        try:
-            revobj = self.branch.repository.get_revision(revid)
-        except bazErrors.NoSuchRevision:
+        if revobj is None:
             # This is a ghost revision. Mark it as not found and next!
             self.revid_to_mark[revid] = -1
             return
-
         # Get the primary parent
         # TODO: Consider the excluded revisions when deciding the parents.
         # Currently, a commit with parents that are excluded ought to be
         # triggering the ref calculation below (and it is not).
         # IGC 20090824
-        ncommits = len(self.revid_to_mark)
-        nparents = len(revobj.parent_ids)
-        if nparents == 0:
+        if len(revobj.parent_ids) == 0:
             parent = breezy.revision.NULL_REVISION
         else:
             parent = revobj.parent_ids[0]
 
+        # Print the commit
+        mark = len(self.revid_to_mark) + 1
+        self.revid_to_mark[revobj.revision_id] = mark
+
+        return [parent, revobj.revision_id]
+
+    def emit_commit(self, revobj, ref, tree_old, tree_new):
         # For parentless commits we need to issue reset command first, otherwise
         # git-fast-import will assume previous commit was this one's parent
-        if nparents == 0:
+        if tree_old.get_revision_id() == breezy.revision.NULL_REVISION:
             self.print_cmd(commands.ResetCommand(ref, None))
 
-        # Print the commit
-        mark = ncommits + 1
-        self.revid_to_mark[revid] = mark
-        file_cmds = self._get_filecommands(parent, revid)
+        file_cmds = self._get_filecommands(tree_old, tree_new)
+        mark = self.revid_to_mark[revobj.revision_id]
         self.print_cmd(self._get_commit_command(ref, mark, revobj, file_cmds))
 
         # Report progress and checkpoint if it's time for that
+        ncommits = len(self.revid_to_mark)
         self.report_progress(ncommits)
         if (self.checkpoint is not None and self.checkpoint > 0 and ncommits and
                 ncommits % self.checkpoint == 0):
@@ -450,81 +474,83 @@ class BzrFastExporter(object):
                     pass
 
         # Build and return the result
-        return commands.CommitCommand(git_ref, mark, author_info,
-                                      committer_info, revobj.message.encode(
-                                          "utf-8"), from_, merges, file_cmds,
-                                      more_authors=more_author_info, properties=properties)
+        return commands.CommitCommand(
+            git_ref, mark, author_info, committer_info,
+            revobj.message.encode("utf-8"), from_, merges, file_cmds,
+            more_authors=more_author_info, properties=properties)
 
-    def _get_revision_trees(self, parent, revision_id):
-        try:
-            tree_old = self.branch.repository.revision_tree(parent)
-        except bazErrors.UnexpectedInventoryFormat:
-            self.warning(
-                "Parent is malformed - diffing against previous parent")
-            # We can't find the old parent. Let's diff against his parent
-            pp = self.branch.repository.get_revision(parent)
-            tree_old = self.branch.repository.revision_tree(pp.parent_ids[0])
-        tree_new = None
-        try:
-            tree_new = self.branch.repository.revision_tree(revision_id)
-        except bazErrors.UnexpectedInventoryFormat:
-            # We can't really do anything anymore
-            self.warning("Revision %s is malformed - skipping" % revision_id)
-        return tree_old, tree_new
+    def _get_revision_trees(self, revids):
+        missing = []
+        by_revid = {}
+        for revid in revids:
+            if revid == breezy.revision.NULL_REVISION:
+                by_revid[revid] = self.branch.repository.revision_tree(revid)
+            elif revid not in self.tree_cache:
+                missing.append(revid)
 
-    def _get_filecommands(self, parent, revision_id):
+        for tree in self.branch.repository.revision_trees(missing):
+            by_revid[tree.get_revision_id()] = tree
+
+        for revid in revids:
+            try:
+                yield self.tree_cache[revid]
+            except KeyError:
+                yield by_revid[revid]
+
+        for revid, tree in by_revid.items():
+            self.tree_cache[revid] = tree
+
+    def _get_filecommands(self, tree_old, tree_new):
         """Get the list of FileCommands for the changes between two revisions."""
-        tree_old, tree_new = self._get_revision_trees(parent, revision_id)
-        if not(tree_old and tree_new):
-            # Something is wrong with this revision - ignore the filecommands
-            return
-
         changes = tree_new.changes_from(tree_old)
 
-        # Make "modified" have 3-tuples, as added does
-        my_modified = [x[0:3] for x in changes.modified]
+        my_modified = list(changes.modified)
 
         # The potential interaction between renames and deletes is messy.
         # Handle it here ...
         file_cmds, rd_modifies, renamed = self._process_renames_and_deletes(
-            changes.renamed, changes.removed, revision_id, tree_old)
+            changes.renamed, changes.removed, tree_new.get_revision_id(), tree_old)
 
         for cmd in file_cmds:
             yield cmd
 
         # Map kind changes to a delete followed by an add
-        for path, id_, kind1, kind2 in changes.kind_changed:
-            path = self._adjust_path_for_renames(path, renamed, revision_id)
+        for change in changes.kind_changed:
+            path = self._adjust_path_for_renames(
+                path, renamed, tree_new.get_revision_id())
             # IGC: I don't understand why a delete is needed here.
             # In fact, it seems harmful? If you uncomment this line,
             # please file a bug explaining why you needed to.
             # yield commands.FileDeleteCommand(path)
-            my_modified.append((path, id_, kind2))
+            my_modified.append(change)
 
         # Record modifications
         files_to_get = []
-        for path, id_, kind in changes.added + my_modified + rd_modifies:
-            if kind == 'file':
+        for change in changes.added + changes.copied + my_modified + rd_modifies:
+            if change.kind[1] == 'file':
                 files_to_get.append(
-                    (path,
-                     (path, helpers.kind_to_mode(
-                         'file', tree_new.is_executable(path)))))
-            elif kind == 'symlink':
+                    (change.path[1],
+                     (change.path[1], helpers.kind_to_mode(
+                         'file', change.executable[1]))))
+            elif change.kind[1] == 'symlink':
                 yield commands.FileModifyCommand(
-                    path.encode("utf-8"),
+                    change.path[1].encode("utf-8"),
                     helpers.kind_to_mode('symlink', False),
-                    None, tree_new.get_symlink_target(path))
-            elif kind == 'directory':
+                    None, tree_new.get_symlink_target(
+                        change.path[1]).encode('utf-8'))
+            elif change.kind[1] == 'directory':
                 if not self.plain_format:
                     yield commands.FileModifyCommand(
-                        path.encode("utf-8"),
+                        change.path[1].encode("utf-8"),
                         helpers.kind_to_mode('directory', False), None,
                         None)
             else:
                 self.warning("cannot export '%s' of kind %s yet - ignoring" %
-                             (path, kind))
-        for (path, mode), chunks in tree_new.iter_files_bytes(
-                files_to_get):
+                             (change.path[1], change.kind[1]))
+
+        # TODO(jelmer): Improve performance on remote repositories
+        # by using Repository.iter_files_bytes for bzr repositories here.
+        for (path, mode), chunks in tree_new.iter_files_bytes(files_to_get):
             yield commands.FileModifyCommand(
                 path.encode("utf-8"), mode, None, b''.join(chunks))
 
@@ -556,37 +582,36 @@ class BzrFastExporter(object):
 
         must_be_renamed = {}
         old_to_new = {}
-        deleted_paths = set([p for p, _, _ in deletes])
-        for (oldpath, newpath, id_, kind,
-                text_modified, meta_modified) in renames:
-            emit = kind != 'directory' or not self.plain_format
-            if newpath in deleted_paths:
+        deleted_paths = set([change.path[0] for change in deletes])
+        for change in renames:
+            emit = change.kind[1] != 'directory' or not self.plain_format
+            if change.path[1] in deleted_paths:
                 if emit:
                     file_cmds.append(commands.FileDeleteCommand(
-                        newpath.encode("utf-8")))
-                deleted_paths.remove(newpath)
-            if (self.is_empty_dir(tree_old, oldpath)):
-                self.note("Skipping empty dir %s in rev %s" % (oldpath,
+                        change.path[1].encode("utf-8")))
+                deleted_paths.remove(change.path[1])
+            if (self.is_empty_dir(tree_old, change.path[0])):
+                self.note("Skipping empty dir %s in rev %s" % (change.path[0],
                                                                revision_id))
                 continue
             # oldpath = self._adjust_path_for_renames(oldpath, renamed,
             #    revision_id)
-            renamed.append([oldpath, newpath])
-            old_to_new[oldpath] = newpath
+            renamed.append(change.path)
+            old_to_new[change.path[0]] = change.path[1]
             if emit:
                 file_cmds.append(
-                    commands.FileRenameCommand(oldpath.encode("utf-8"), newpath.encode("utf-8")))
-            if text_modified or meta_modified:
-                modifies.append((newpath, id_, kind))
+                    commands.FileRenameCommand(change.path[0].encode("utf-8"), change.path[1].encode("utf-8")))
+            if change.changed_content or change.meta_modified():
+                modifies.append(change)
 
             # Renaming a directory implies all children must be renamed.
             # Note: changes_from() doesn't handle this
-            if kind == 'directory' and tree_old.kind(oldpath) == 'directory':
-                for p, e in tree_old.iter_entries_by_dir(specific_files=[oldpath]):
+            if change.kind == ('directory', 'directory'):
+                for p, e in tree_old.iter_entries_by_dir(specific_files=[change.path[0]]):
                     if e.kind == 'directory' and self.plain_format:
                         continue
-                    old_child_path = osutils.pathjoin(oldpath, p)
-                    new_child_path = osutils.pathjoin(newpath, p)
+                    old_child_path = osutils.pathjoin(change.path[0], p)
+                    new_child_path = osutils.pathjoin(change.path[1], p)
                     must_be_renamed[old_child_path] = new_child_path
 
         # Add children not already renamed
@@ -602,13 +627,13 @@ class BzrFastExporter(object):
                                                             new_child_path.encode("utf-8")))
 
         # Record remaining deletes
-        for path, id_, kind in deletes:
-            if path not in deleted_paths:
+        for change in deletes:
+            if change.path[0] not in deleted_paths:
                 continue
-            if kind == 'directory' and self.plain_format:
+            if change.kind[0] == 'directory' and self.plain_format:
                 continue
             #path = self._adjust_path_for_renames(path, renamed, revision_id)
-            file_cmds.append(commands.FileDeleteCommand(path.encode("utf-8")))
+            file_cmds.append(commands.FileDeleteCommand(change.path[0].encode("utf-8")))
         return file_cmds, modifies, renamed
 
     def _adjust_path_for_renames(self, path, renamed, revision_id):
