@@ -20,10 +20,11 @@ import errno
 import os
 
 from ... import (
-    branch,
+    branch as _mod_branch,
     config,
     controldir,
     errors,
+    merge,
     osutils,
     revision as _mod_revision,
     tests,
@@ -79,7 +80,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
     def test_branch_builder(self):
         # Just a smoke test that we get a branch at the specified relpath
         builder = self.make_branch_builder('foobar')
-        br = branch.Branch.open(self.get_url('foobar'))
+        br = _mod_branch.Branch.open(self.get_url('foobar'))
 
     def test_list_files(self):
         tree = self.make_branch_and_tree('.')
@@ -142,6 +143,11 @@ class TestWorkingTree(TestCaseWithWorkingTree):
                     ('zz_dir', 'directory'),
                     ], files)
 
+    def test_get_transform(self):
+        tree = self.make_branch_and_tree('tree')
+        with tree.get_transform():
+            pass
+
     def test_list_files_kind_change(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/filename'])
@@ -154,14 +160,12 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         self.assertEqual(1, len(result))
         if tree.has_versioned_directories():
             self.assertEqual(
-                ('filename', 'V', 'directory', tree.path2id('filename')),
-                (result[0][0], result[0][1], result[0][2],
-                    getattr(result[0][3], 'file_id', None)))
+                ('filename', 'V', 'directory'),
+                (result[0][0], result[0][1], result[0][2]))
         else:
             self.assertEqual(
-                ('filename', '?', 'directory', None),
-                (result[0][0], result[0][1], result[0][2],
-                    getattr(result[0][3], 'file_id', None)))
+                ('filename', '?', 'directory'),
+                (result[0][0], result[0][1], result[0][2]))
 
     def test_get_config_stack(self):
         # Smoke test that all working trees succeed getting a config
@@ -296,7 +300,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
     def test_initialize(self):
         # initialize should create a working tree and branch in an existing dir
         t = self.make_branch_and_tree('.')
-        b = branch.Branch.open('.')
+        b = _mod_branch.Branch.open('.')
         self.assertEqual(t.branch.base, b.base)
         t2 = WorkingTree.open('.')
         self.assertEqual(t.basedir, t2.basedir)
@@ -570,8 +574,8 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         # and update checkout
         self.assertEqual(0, checkout.update())
         self.assertPathExists('checkout/file')
-        self.assertEqual(wt.get_root_id(), checkout.get_root_id())
-        self.assertNotEqual(None, wt.get_root_id())
+        self.assertEqual(wt.path2id(''), checkout.path2id(''))
+        self.assertNotEqual(None, wt.path2id(''))
 
     def test_update_sets_updated_root_id(self):
         wt = self.make_branch_and_tree('tree')
@@ -580,16 +584,16 @@ class TestWorkingTree(TestCaseWithWorkingTree):
                               'first_root_id')
             return
         wt.set_root_id(b'first_root_id')
-        self.assertEqual(b'first_root_id', wt.get_root_id())
+        self.assertEqual(b'first_root_id', wt.path2id(''))
         self.build_tree(['tree/file'])
         wt.add(['file'])
         wt.commit('first')
         co = wt.branch.create_checkout('checkout')
         wt.set_root_id(b'second_root_id')
         wt.commit('second')
-        self.assertEqual(b'second_root_id', wt.get_root_id())
+        self.assertEqual(b'second_root_id', wt.path2id(''))
         self.assertEqual(0, co.update())
-        self.assertEqual(b'second_root_id', co.get_root_id())
+        self.assertEqual(b'second_root_id', co.path2id(''))
 
     def test_update_returns_conflict_count(self):
         # working tree formats from the meta-dir format and newer support
@@ -731,7 +735,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         self.build_tree_contents([('tree/somefile', b'hello')])
         with tree.lock_write():
             tree.add(['somefile'])
-            d = {tree.path2id('somefile'): osutils.sha_string(b'hello')}
+            d = {'somefile': osutils.sha_string(b'hello')}
             if tree.supports_merge_modified():
                 tree.set_merge_modified(d)
                 mm = tree.merge_modified()
@@ -915,7 +919,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         # if we write write an inventory then do a walkdirs we should get back
         # missing entries, and actual, and unknowns as appropriate.
         self.build_tree(['present', 'unknown'])
-        inventory = Inventory(tree.get_root_id())
+        inventory = Inventory(tree.path2id(''))
         inventory.add_path('missing', 'file', b'missing-id')
         inventory.add_path('present', 'file', b'present-id')
         # there is no point in being able to write an inventory to an unlocked
@@ -927,7 +931,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
             present_stat = os.lstat('present')
             unknown_stat = os.lstat('unknown')
             expected_results = [
-                (('', tree.get_root_id()),
+                (('', tree.path2id('')),
                  [('missing', 'missing', 'unknown', None, b'missing-id', 'file'),
                   ('present', 'present', 'file',
                    present_stat, b'present-id', 'file'),
@@ -1326,3 +1330,225 @@ class TestFormatAttributes(TestCaseWithWorkingTree):
         self.assertSubset(
             [self.workingtree_format.supports_store_uncommitted],
             (True, False))
+
+
+class TestReferenceLocation(TestCaseWithWorkingTree):
+
+    def test_reference_parent(self):
+        tree = self.make_branch_and_tree('tree')
+        subtree = self.make_branch_and_tree('tree/subtree')
+        subtree.commit('a change')
+        try:
+            tree.add_reference(subtree)
+        except errors.UnsupportedOperation:
+            raise tests.TestNotApplicable('Tree cannot hold references.')
+        if not getattr(tree.branch._format, 'supports_reference_locations', False):
+            raise tests.TestNotApplicable('Branch cannot hold reference locations.')
+        tree.commit('Add reference.')
+        reference_parent = tree.reference_parent(
+            urlutils.relative_url(
+                urlutils.strip_segment_parameters(tree.branch.user_url),
+                urlutils.strip_segment_parameters(subtree.branch.user_url)))
+        self.assertEqual(subtree.branch.user_url, reference_parent.user_url)
+
+    def test_reference_parent_accepts_possible_transports(self):
+        tree = self.make_branch_and_tree('tree')
+        subtree = self.make_branch_and_tree('tree/subtree')
+        subtree.commit('a change')
+        try:
+            tree.add_reference(subtree)
+        except errors.UnsupportedOperation:
+            raise tests.TestNotApplicable('Tree cannot hold references.')
+        if not getattr(tree.branch._format, 'supports_reference_locations', False):
+            raise tests.TestNotApplicable('Branch cannot hold reference locations.')
+        tree.commit('Add reference')
+        reference_parent = tree.reference_parent(
+            urlutils.relative_url(
+                urlutils.strip_segment_parameters(tree.branch.user_url),
+                urlutils.strip_segment_parameters(subtree.branch.user_url)),
+            possible_transports=[subtree.controldir.root_transport])
+
+    def test_get_reference_info(self):
+        tree = self.make_branch_and_tree('branch')
+        try:
+            loc = tree.get_reference_info('file')
+        except errors.UnsupportedOperation:
+            raise tests.TestNotApplicable('Branch cannot hold references.')
+        self.assertIs(None, loc)
+
+    def test_set_reference_info(self):
+        self.make_tree_with_reference('branch', 'path/to/location')
+
+    def test_set_get_reference_info(self):
+        tree = self.make_tree_with_reference('branch', 'path/to/location')
+        # Create a new instance to ensure storage is permanent
+        tree = WorkingTree.open('branch')
+        branch_location = tree.get_reference_info('path/to/file')
+        self.assertEqual(
+            urlutils.join(urlutils.strip_segment_parameters(tree.branch.user_url), 'path/to/location'),
+            urlutils.join(urlutils.strip_segment_parameters(tree.branch.user_url), branch_location))
+
+    def test_set_null_reference_info(self):
+        tree = self.make_branch_and_tree('branch')
+        self.build_tree(['branch/file'])
+        tree.add(['file'])
+        try:
+            tree.set_reference_info('file', 'path/to/location')
+        except errors.UnsupportedOperation:
+            raise tests.TestNotApplicable('Branch cannot hold references.')
+        tree.set_reference_info('file', None)
+        branch_location = tree.get_reference_info('file')
+        self.assertIs(None, branch_location)
+
+    def test_set_null_reference_info_when_null(self):
+        tree = self.make_branch_and_tree('branch')
+        try:
+            branch_location = tree.get_reference_info('file')
+        except errors.UnsupportedOperation:
+            raise tests.TestNotApplicable('Branch cannot hold references.')
+        self.assertIs(None, branch_location)
+        self.build_tree(['branch/file'])
+        tree.add(['file'])
+        try:
+            tree.set_reference_info('file', None)
+        except errors.UnsupportedOperation:
+            raise tests.TestNotApplicable('Branch cannot hold references.')
+
+    def make_tree_with_reference(self, location, reference_location):
+        tree = self.make_branch_and_tree(location)
+        self.build_tree(
+            [os.path.join(location, name)
+             for name in ['path/', 'path/to/', 'path/to/file']])
+        tree.add(['path', 'path/to', 'path/to/file'])
+        try:
+            tree.set_reference_info('path/to/file', reference_location)
+        except errors.UnsupportedOperation:
+            raise tests.TestNotApplicable('Branch cannot hold references.')
+        tree.commit('commit reference')
+        return tree
+
+    def test_reference_parent_from_reference_info_(self):
+        referenced_branch = self.make_branch('reference_branch')
+        tree = self.make_tree_with_reference('branch', referenced_branch.base)
+        parent = tree.reference_parent('path/to/file')
+        self.assertEqual(parent.base, referenced_branch.base)
+
+    def test_branch_relative_reference_location(self):
+        tree = self.make_tree_with_reference('branch', '../reference_branch')
+        referenced_branch = self.make_branch('reference_branch')
+        parent = tree.reference_parent('path/to/file')
+        self.assertEqual(parent.base, referenced_branch.base)
+
+    def test_sprout_copies_reference_location(self):
+        tree = self.make_tree_with_reference('branch', '../reference')
+        new_tree = tree.branch.controldir.sprout('new-branch').open_workingtree()
+        self.assertEqual(
+            urlutils.join(urlutils.strip_segment_parameters(tree.branch.user_url),
+                          '../reference'),
+            urlutils.join(urlutils.strip_segment_parameters(new_tree.branch.user_url),
+                          new_tree.get_reference_info('path/to/file')))
+
+    def test_clone_copies_reference_location(self):
+        tree = self.make_tree_with_reference('branch', '../reference')
+        new_tree = tree.controldir.clone('new-branch').open_workingtree()
+        self.assertEqual(
+            urlutils.join(urlutils.strip_segment_parameters(tree.branch.user_url), '../reference'),
+            urlutils.join(urlutils.strip_segment_parameters(new_tree.branch.user_url),
+                          new_tree.get_reference_info('path/to/file')))
+
+    def test_copied_locations_are_rebased(self):
+        tree = self.make_tree_with_reference('branch', 'reference')
+        new_tree = tree.controldir.sprout(
+            'branch/new-branch').open_workingtree()
+        self.assertEqual(
+            urlutils.join(urlutils.strip_segment_parameters(tree.branch.user_url),
+                          'reference'),
+            urlutils.join(urlutils.strip_segment_parameters(new_tree.branch.user_url),
+                          new_tree.get_reference_info('path/to/file')))
+
+    def test_update_references_retains_old_references(self):
+        tree = self.make_tree_with_reference('branch', 'reference')
+        new_tree = self.make_tree_with_reference(
+            'new_branch', 'reference2')
+        new_tree.branch.update_references(tree.branch)
+        self.assertEqual(
+            urlutils.join(
+                urlutils.strip_segment_parameters(tree.branch.user_url),
+                'reference'),
+            urlutils.join(
+                urlutils.strip_segment_parameters(tree.branch.user_url),
+                tree.get_reference_info('path/to/file')))
+
+    def test_update_references_retains_known_references(self):
+        tree = self.make_tree_with_reference('branch', 'reference')
+        new_tree = self.make_tree_with_reference(
+            'new_branch', 'reference2')
+        new_tree.branch.update_references(tree.branch)
+        self.assertEqual(
+            urlutils.join(
+                urlutils.strip_segment_parameters(tree.branch.user_url),
+                'reference'),
+            urlutils.join(
+                urlutils.strip_segment_parameters(tree.branch.user_url),
+                tree.get_reference_info('path/to/file')))
+
+    def test_update_references_skips_known_references(self):
+        tree = self.make_tree_with_reference('branch', 'reference')
+        new_tree = tree.controldir.sprout(
+            'branch/new-branch').open_workingtree()
+        self.build_tree(['branch/new-branch/foo'])
+        new_tree.add('foo')
+        new_tree.set_reference_info('foo', '../foo')
+        new_tree.branch.update_references(tree.branch)
+        self.assertEqual(
+            urlutils.join(
+                urlutils.strip_segment_parameters(tree.branch.user_url),
+                'reference'),
+            urlutils.join(
+                urlutils.strip_segment_parameters(tree.branch.user_url),
+                tree.get_reference_info('path/to/file')))
+
+    def test_pull_updates_references(self):
+        tree = self.make_tree_with_reference('branch', 'reference')
+        new_tree = tree.controldir.sprout(
+            'branch/new-branch').open_workingtree()
+        self.build_tree(['branch/new-branch/foo'])
+        new_tree.add('foo')
+        new_tree.set_reference_info('foo', '../foo')
+        new_tree.commit('set reference')
+        tree.pull(new_tree.branch)
+        self.assertEqual(
+            urlutils.join(urlutils.strip_segment_parameters(new_tree.branch.user_url), '../foo'),
+            urlutils.join(tree.branch.user_url, tree.get_reference_info('foo')))
+
+    def test_push_updates_references(self):
+        tree = self.make_tree_with_reference('branch', 'reference')
+        new_tree = tree.controldir.sprout(
+            'branch/new-branch').open_workingtree()
+        self.build_tree(['branch/new-branch/foo'])
+        new_tree.add(['foo'])
+        new_tree.set_reference_info('foo', '../foo')
+        new_tree.commit('add reference')
+        tree.pull(new_tree.branch)
+        tree.update()
+        self.assertEqual(
+            urlutils.join(urlutils.strip_segment_parameters(new_tree.branch.user_url), '../foo'),
+            urlutils.join(urlutils.strip_segment_parameters(tree.branch.user_url), tree.get_reference_info('foo')))
+
+    def test_merge_updates_references(self):
+        orig_tree = self.make_tree_with_reference('branch', 'reference')
+        tree = orig_tree.controldir.sprout('tree').open_workingtree()
+        tree.commit('foo')
+        orig_tree.pull(tree.branch)
+        checkout = orig_tree.branch.create_checkout('checkout', lightweight=True)
+        checkout.commit('bar')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        merger = merge.Merger.from_revision_ids(tree,
+                                                orig_tree.branch.last_revision(),
+                                                other_branch=orig_tree.branch)
+        merger.merge_type = merge.Merge3Merger
+        merger.do_merge()
+        self.assertEqual(
+            urlutils.join(urlutils.strip_segment_parameters(orig_tree.branch.user_url), 'reference'),
+            urlutils.join(urlutils.strip_segment_parameters(tree.branch.user_url), tree.get_reference_info('path/to/file')))
