@@ -821,6 +821,20 @@ class ChrootedTests(TestCaseWithTransport):
         self.assertEqual(os.path.realpath('topdir/foo'),
                          self.local_branch_path(branch))
 
+    def test_open_tree_or_branch_named(self):
+        tree = self.make_branch_and_tree('topdir')
+        self.assertRaises(
+            NotBranchError,
+            bzrdir.BzrDir.open_tree_or_branch, 'topdir', name='missing')
+        tree.branch.controldir.create_branch('named')
+        tree, branch = bzrdir.BzrDir.open_tree_or_branch('topdir', name='named')
+        self.assertEqual(os.path.realpath('topdir'),
+                         os.path.realpath(tree.basedir))
+        self.assertEqual(os.path.realpath('topdir'),
+                         self.local_branch_path(branch))
+        self.assertEqual(branch.name, 'named')
+        self.assertIs(tree.controldir, branch.controldir)
+
     def test_open_from_transport(self):
         # transport pointing at bzrdir should give a bzrdir with root transport
         # set to the given transport
@@ -842,12 +856,11 @@ class ChrootedTests(TestCaseWithTransport):
         self.assertRaises(NotBranchError, bzrdir.BzrDir.open_from_transport, t)
 
     def test_sprout_recursive(self):
-        tree = self.make_branch_and_tree('tree1',
-                                         format='development-subtree')
-        sub_tree = self.make_branch_and_tree('tree1/subtree',
-                                             format='development-subtree')
+        tree = self.make_branch_and_tree('tree1')
+        sub_tree = self.make_branch_and_tree('tree1/subtree')
         sub_tree.set_root_id(b'subtree-root')
         tree.add_reference(sub_tree)
+        tree.set_reference_info('subtree', sub_tree.branch.user_url)
         self.build_tree(['tree1/subtree/file'])
         sub_tree.add('file')
         tree.commit('Initial commit')
@@ -872,12 +885,12 @@ class ChrootedTests(TestCaseWithTransport):
         sub_tree = self.make_branch_and_tree('tree1/subtree',
                                              format='development-subtree')
         tree.add_reference(sub_tree)
+        tree.set_reference_info('subtree', sub_tree.branch.user_url)
         self.build_tree(['tree1/subtree/file'])
         sub_tree.add('file')
         tree.commit('Initial commit')
         # The following line force the orhaning to reveal bug #634470
-        tree.branch.get_config_stack().set(
-            'transform.orphan_policy', 'move')
+        tree.branch.get_config_stack().set('transform.orphan_policy', 'move')
         tree.controldir.destroy_workingtree()
         # FIXME: subtree/.bzr is left here which allows the test to pass (or
         # fail :-( ) -- vila 20100909
@@ -892,10 +905,9 @@ class ChrootedTests(TestCaseWithTransport):
         # by bzrdir.BzrDirMeta1.destroy_workingtree when it ignores the
         # [DeletingParent('Not deleting', u'subtree', None)] conflict). See bug
         # #634470.  -- vila 20100909
-        self.assertRaises(errors.NotBranchError,
-                          tree.controldir.sprout, 'repo/tree2')
-#        self.assertPathExists('repo/tree2/subtree')
-#        self.assertPathDoesNotExist('repo/tree2/subtree/file')
+        tree.controldir.sprout('repo/tree2')
+        self.assertPathExists('repo/tree2/subtree')
+        self.assertPathDoesNotExist('repo/tree2/subtree/file')
 
     def make_foo_bar_baz(self):
         foo = bzrdir.BzrDir.create_branch_convenience('foo').controldir
@@ -1240,17 +1252,11 @@ class TestDotBzrHidden(TestCaseWithTransport):
         return out.splitlines()
 
     def test_dot_bzr_hidden(self):
-        if sys.platform == 'win32' and not win32utils.has_win32file:
-            raise TestSkipped(
-                'unable to make file hidden without pywin32 library')
         b = bzrdir.BzrDir.create('.')
         self.build_tree(['a'])
         self.assertEqual([b'a'], self.get_ls())
 
     def test_dot_bzr_hidden_with_url(self):
-        if sys.platform == 'win32' and not win32utils.has_win32file:
-            raise TestSkipped(
-                'unable to make file hidden without pywin32 library')
         b = bzrdir.BzrDir.create(urlutils.local_path_to_url('.'))
         self.build_tree(['a'])
         self.assertEqual([b'a'], self.get_ls())
