@@ -39,9 +39,12 @@ Some properties are common to all kinds:
 
 try:
     import http.client as http_client
-    parse_headers = http_client.parse_headers
-except ImportError:  # python < 3
+except ImportError:  # python < 3 without future
     import httplib as http_client
+
+try:
+    parse_headers = http_client.parse_headers
+except AttributeError:  # python 2
     parse_headers = http_client.HTTPMessage
 
 from .. import (
@@ -50,10 +53,11 @@ from .. import (
     )
 from ..sixish import (
     BytesIO,
+    PY3,
     )
 from ..transport.http import (
     response,
-    _urllib2_wrappers,
+    HTTPConnection,
     )
 from .file_utils import (
     FakeReadFile,
@@ -70,10 +74,10 @@ class ReadSocket(object):
         return self.readfile
 
 
-class FakeHTTPConnection(_urllib2_wrappers.HTTPConnection):
+class FakeHTTPConnection(HTTPConnection):
 
     def __init__(self, sock):
-        _urllib2_wrappers.HTTPConnection.__init__(self, 'localhost')
+        HTTPConnection.__init__(self, 'localhost')
         # Set the socket to bypass the connection
         self.sock = sock
 
@@ -722,14 +726,17 @@ class TestHandleResponse(tests.TestCase):
         # Get rid of the status line
         status_and_headers.readline()
         msg = parse_headers(status_and_headers)
-        return msg
+        if PY3:
+            return msg.get
+        else:
+            return msg.getheader
 
     def get_response(self, a_response):
         """Process a supplied response, and return the result."""
         code, raw_headers, body = a_response
-        msg = self._build_HTTPMessage(raw_headers)
-        return response.handle_response('http://foo', code, msg,
-                                        BytesIO(a_response[2]))
+        getheader = self._build_HTTPMessage(raw_headers)
+        return response.handle_response(
+            'http://foo', code, getheader, BytesIO(a_response[2]))
 
     def test_full_text(self):
         out = self.get_response(_full_text_response)
@@ -780,29 +787,31 @@ class TestHandleResponse(tests.TestCase):
     def test_full_text_no_content_type(self):
         # We should not require Content-Type for a full response
         code, raw_headers, body = _full_text_response_no_content_type
-        msg = self._build_HTTPMessage(raw_headers)
-        out = response.handle_response('http://foo', code, msg, BytesIO(body))
+        getheader = self._build_HTTPMessage(raw_headers)
+        out = response.handle_response(
+            'http://foo', code, getheader, BytesIO(body))
         self.assertEqual(body, out.read())
 
     def test_full_text_no_content_length(self):
         code, raw_headers, body = _full_text_response_no_content_length
-        msg = self._build_HTTPMessage(raw_headers)
-        out = response.handle_response('http://foo', code, msg, BytesIO(body))
+        getheader = self._build_HTTPMessage(raw_headers)
+        out = response.handle_response(
+            'http://foo', code, getheader, BytesIO(body))
         self.assertEqual(body, out.read())
 
     def test_missing_content_range(self):
         code, raw_headers, body = _single_range_no_content_range
-        msg = self._build_HTTPMessage(raw_headers)
+        getheader = self._build_HTTPMessage(raw_headers)
         self.assertRaises(errors.InvalidHttpResponse,
                           response.handle_response,
-                          'http://bogus', code, msg, BytesIO(body))
+                          'http://bogus', code, getheader, BytesIO(body))
 
     def test_multipart_no_content_range(self):
         code, raw_headers, body = _multipart_no_content_range
-        msg = self._build_HTTPMessage(raw_headers)
+        getheader = self._build_HTTPMessage(raw_headers)
         self.assertRaises(errors.InvalidHttpResponse,
                           response.handle_response,
-                          'http://bogus', code, msg, BytesIO(body))
+                          'http://bogus', code, getheader, BytesIO(body))
 
     def test_multipart_no_boundary(self):
         out = self.get_response(_multipart_no_boundary)
